using System;
using System.Collections;
using System.Configuration;
using System.IO;
using System.Reflection;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Lifetime;
using System.Runtime.Remoting.Channels;
using System.Threading;
using System.Windows.Forms;

using KnownObjects;

using Belikov.GenuineChannels;
using Belikov.GenuineChannels.Logbook;
using Belikov.GenuineChannels.DotNetRemotingLayer;
using Belikov.GenuineChannels.BroadcastEngine;
using Belikov.GenuineChannels.GenuineUdp;

namespace Client
{
	/// <summary>
	/// Server.
	/// </summary>
	class Class1 : MarshalByRefObject, IDownloadConfirmation
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			try
			{
				Console.WriteLine("Sleep for 3 seconds.");
				Thread.Sleep(TimeSpan.FromSeconds(3));

				// initialize Remoting
				System.Configuration.ConfigurationSettings.GetConfig("DNS");
				GenuineGlobalEventProvider.GenuineChannelsGlobalEvent += new GenuineChannelsGlobalEventHandler(GenuineChannelsEventHandler);

				// configuration via config
				RemotingConfiguration.Configure("Client.exe.config");
				Console.WriteLine(".NET Remoting environment has been initialized.");

				IContentDownloader iContentDownloader = (IContentDownloader) Activator.GetObject(typeof(IContentDownloader), 
					ConfigurationSettings.AppSettings["RemoteHostUri"] + "ContentDownloader.rem");


				// let user to choose file to upload
				OpenFileDialog openFileDialog = new OpenFileDialog();
				if (openFileDialog.ShowDialog() == DialogResult.OK)
				{
					Stream fileStream = openFileDialog.OpenFile();
					iContentDownloader.DownloadContent(new ProviderBasedOnStream(fileStream), (int) fileStream.Length, new Class1());
					Console.WriteLine("Download initiated.");
				}

				_fileDownloaded.WaitOne();

				Console.WriteLine("Download finished.");
			}
			catch(Exception ex)
			{
				Console.WriteLine("Exception: {0}.", ex.Message);
			}
		}

		private static ManualResetEvent _fileDownloaded = new ManualResetEvent(false);

		/// <summary>
		/// Is called by the remote host to inform of the download progress.
		/// </summary>
		/// <param name="percent">Percent downloaded.</param>
		public void Progress(int percent)
		{
			Console.WriteLine("Progress: {0}%.", percent);
		}

		/// <summary>
		/// Is called by the remote host to inform of the download progress.
		/// </summary>
		/// <param name="percent">Percent downloaded.</param>
		public void OneWayProgress(int percent)
		{
			this.Progress(percent);
		}

		public void Downloaded()
		{
			_fileDownloaded.Set();
		}

		public static void GenuineChannelsEventHandler(object sender, GenuineEventArgs e)
		{
			if (e.SourceException == null)
				Console.WriteLine("Global event: {0}\r\nUrl: {1}", e.EventType, 
					e.HostInformation == null ? "<not specified>" : e.HostInformation.ToString());
			else
				Console.WriteLine("Global event: {0}\r\nUrl: {1}\r\nException: {2}", e.EventType, 
					e.HostInformation == null ? "<not specified>" : e.HostInformation.ToString(), 
					e.SourceException);
		}
	}
}
