using System;
using System.IO;

using KnownObjects;

namespace Client
{
	/// <summary>
	/// ProviderBasedOnStream.
	/// </summary>
	public class ProviderBasedOnStream : MarshalByRefObject, IDownloadableContent
	{
		/// <summary>
		/// Constructs ProviderBasedOnStream instance.
		/// </summary>
		/// <param name="stream">Stream.</param>
		public ProviderBasedOnStream(Stream stream)
		{
			this.Stream = stream;
		}

		public Stream Stream;

		/// <summary>
		/// Is used to get the next portion of the data being downloaded.
		/// </summary>
		/// <param name="portionSize">Size of the requested portion.</param>
		/// <returns>Portion.</returns>
		public byte[] GetNextPortion(int portionSize)
		{
			// allocate a chunk
			portionSize = Math.Min((int) (this.Stream.Length - this.Stream.Position), portionSize);
			byte[] chunk = new byte[portionSize];

			// read from the stream
			portionSize = this.Stream.Read(chunk, 0, chunk.Length);

			if (portionSize != chunk.Length)
			{
				// in my case its impossible, but probably this code will fit to your needs
				byte[] secondChunk = new byte[portionSize];
				Buffer.BlockCopy(chunk, 0, secondChunk, 0, portionSize);
				return secondChunk;
			}

			return chunk;
		}
	}
}
