using System;
using System.IO;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Messaging;

namespace KnownObjects
{
	/// <summary>
	/// Downloader interface.
	/// </summary>
	public interface IContentDownloader
	{
		/// <summary>
		/// Requests a server to download a content.
		/// </summary>
		/// <param name="content">The content to upload.</param>
		/// <param name="totalSize">Total content size.</param>
		/// <param name="confirmation">Confirmation agent.</param>
		void DownloadContent(IDownloadableContent content, int totalSize, IDownloadConfirmation confirmation);
	}

	/// <summary>
	/// Represents downloadable content.
	/// </summary>
	public interface IDownloadableContent
	{
		/// <summary>
		/// Is used to get the next portion of the data being downloaded.
		/// </summary>
		/// <param name="portionSize">Size of the requested portion.</param>
		/// <returns>Portion.</returns>
		byte[] GetNextPortion(int portionSize);
	}

	/// <summary>
	/// Is used by the server to inform about download progress.
	/// </summary>
	public interface IDownloadConfirmation
	{
		/// <summary>
		/// Is called by the remote host to inform of the download progress.
		/// </summary>
		/// <param name="percent">Percent downloaded.</param>
		void Progress(int percent);

		/// <summary>
		/// Is called by the remote host to inform of the download progress.
		/// </summary>
		/// <param name="percent">Percent downloaded.</param>
		[OneWay]
		void OneWayProgress(int percent);

		/// <summary>
		/// Is called after downloading is finished.
		/// </summary>
		void Downloaded();
	}
}
