using System;
using System.IO;
using System.Collections;
using System.Reflection;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Activation;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Lifetime;
using System.Runtime.Remoting.Messaging;
using System.Net;
using System.Threading;

using KnownObjects;
using Belikov.GenuineChannels;
using Belikov.GenuineChannels.DotNetRemotingLayer;
using Belikov.GenuineChannels.Messaging;
using Belikov.GenuineChannels.BroadcastEngine;
using Belikov.GenuineChannels.GenuineUdp;

namespace Server
{
	/// <summary>
	/// Server.
	/// </summary>
	class Server : MarshalByRefObject, IContentDownloader
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			Random random = new Random();

			// initialize Remoting
			System.Configuration.ConfigurationSettings.GetConfig("DNS");
			GenuineGlobalEventProvider.GenuineChannelsGlobalEvent += new GenuineChannelsGlobalEventHandler(GenuineChannelsEventHandler);

			// configuration via .config file
			RemotingConfiguration.Configure("Server.exe.config");

			RemotingServices.Marshal(new Server(), "ContentDownloader.rem");

			Console.WriteLine(".NET Remoting environment has been initialized.");

			Console.WriteLine("Press ENTER to exit.");
			Console.ReadLine();
		}

		/// <summary>
		/// Request to the server to upload a content.
		/// </summary>
		/// <param name="content">The content to upload.</param>
		/// <param name="totalSize">Total content size.</param>
		/// <param name="confirmation">Confirmation agent.</param>
		public void DownloadContent(IDownloadableContent content, int totalSize, IDownloadConfirmation confirmation)
		{
			// fork download thread
			new Downloader(content, totalSize, confirmation);
		}

		public static void GenuineChannelsEventHandler(object sender, GenuineEventArgs e)
		{
			if (e.SourceException == null)
				Console.WriteLine("Global event: {0}\r\nUrl: {1}", e.EventType, 
					e.HostInformation == null ? "<not specified>" : e.HostInformation.ToString());
			else
				Console.WriteLine("Global event: {0}\r\nUrl: {1}\r\nException: {2}", e.EventType, 
					e.HostInformation == null ? "<not specified>" : e.HostInformation.ToString(), 
					e.SourceException);
		}

	}
}
